#
# Copyright (c) 2018 Joris Vink <joris@coders.se>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

#
# Asynchronous queue example.
#

import kore

# The shared queue.
tq = kore.queue()

# Entry point for our independent coroutine that is created when kore starts.
async def queue_helper():
    while True:
        # Wait for a dictionary to arrive.
        obj = await tq.pop()
        kore.log(kore.LOG_INFO, "coro(): received %s" % obj)

        # Create a message to send back.
        msg = "%d = %s" % (kore.time(), obj["msg"])

        # Send it on the received queue.
        obj["rq"].push(msg)

async def async_queue(req):
    # Create our own queue.
    rq = kore.queue()

    # The dictionary we are going to send.
    obj = {
        # Receive queue object.
        "rq": rq,
        "msg": "hello"
    }

    # Push it onto the tq queue now, which will wake up the other coroutine.
    tq.push(obj)

    # Wait for a response.
    response = await rq.pop()

    # Send the response to the client.
    req.response(200, response.encode())



#
# Copyright (c) 2018 Joris Vink <joris@coders.se>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

#
# Asynchronous process example.
#
# Wait for the result of an external process asynchronously.
# The handler will execute "/bin/ls" on the current directory and
# read the result.
#

import kore
import json

async def async_proc(req):
    #
    # You may specify a timeout when creating the kore.proc object.
    # If the timeout is reached before the process exits kore will
    # raise a TimeoutError exception.
    #
    # Ex: set timeout to 100ms:
    #   proc = kore.proc("/bin/ls -lR", 100)

    proc = kore.proc("/bin/ls -lR")

    try:
        stdout = ""

        # Read until EOF (None is returned)
        while True:
            try:
                # Read from the process, with an optional 1 second timeout.
                # The recv() call will throw a TimeoutError exception if
                # the timeout has elapsed before any data was read.
                chunk = await proc.recv(1024, 1000)
                if chunk is None:
                    break
            except TimeoutError as e:
                print("recv() timed out: %s" % e)
                continue
            stdout += chunk.decode()

        # Reap the process.
        retcode = await proc.reap()

        # Respond with the return code + the result as JSON.
        payload = {
            "retcode": retcode,
            "stdout": stdout
        }

        data = json.dumps(payload, indent=4)
        req.response(200, data.encode())
    except Exception as e:
        # If an exception occurs we must kill the process first.
        proc.kill()
        errmsg = "Exception: %s" % e
        req.response(500, errmsg.encode())



#
# Copyright (c) 2017-2018 Joris Vink <joris@coders.se>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

# Asynchronous postgresql queries with Python.

import json
import kore

class KoreApp:
    def configure(self, args):
        # Register the path to our database when Kore starts.
        kore.dbsetup("db", "host=/tmp dbname=test")

    # A handler that returns 200 OK with hello as body.
    def hello(self, req):
        req.response(200, b'hello\n')

    #
    # The query handler that fires of the query and returns a coroutine.
    #
    # Kore will resume this handler when the query returns a result or
    # is successful.
    #
    # The kore.pgsql() method can throw exceptions, most notably a
    # GeneratorExit in case the client connection went away before
    # the query was able to be completed.
    #
    # In this example we're not doing any exception handling.
    #
    async def query(self, req):
        result = await kore.dbquery("db", "SELECT * FROM coders")
        req.response(200, json.dumps(result).encode("utf-8"))

    #
    # A slow query that returns after 10 seconds.
    #
    async def slow(self, req):
        result = await kore.dbquery("db", "SELECT * FROM pg_sleep(10)")
        req.response(200, json.dumps(result).encode("utf-8"))

# Set the application Kore will run to our class.
koreapp = KoreApp()



#
# Copyright (c) 2019 Joris Vink <joris@coders.se>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

#
# Asynchronous HTTP client example.
#

import kore

# Handler called for /httpclient
async def httpclient(req):
    # Create an httpclient.
    client = kore.httpclient("https://kore.io")

    # Do a simple GET request.
    print("firing off request")
    status, body = await client.get()
    print("status: %d, body: '%s'" % (status, body))

    # Reuse and perform another GET request, returning headers too this time.
    status, headers, body = await client.get(return_headers=True)
    print("status: %d, headers: '%s'" % (status, headers))

    # What happens if we post something?
    status, body = await client.post(body=b"hello world")
    print("status: %d, body: '%s'" % (status, body))

    # Add some custom headers to our requests.
    status, body = await client.get(
        headers={
            "x-my-header": "async-http"
        }
    )

    req.response(200, b'async done')
